@@ -491,18 +491,48 @@ g_tls_client_connection_gnutls_prepare_handshake (GTlsConnectionBase  *tls,
 
 static void
 g_tls_client_connection_gnutls_complete_handshake (GTlsConnectionBase  *tls,
+                                                   gboolean             handshake_succeeded,
                                                    gchar              **negotiated_protocol,
                                                    GError             **error)
 {
   GTlsClientConnectionGnutls *gnutls = G_TLS_CLIENT_CONNECTION_GNUTLS (tls);
+  gnutls_session_t session;
+  gnutls_protocol_t version;
 
-  G_TLS_CONNECTION_BASE_CLASS (g_tls_client_connection_gnutls_parent_class)->complete_handshake (tls, negotiated_protocol, error);
+  G_TLS_CONNECTION_BASE_CLASS (g_tls_client_connection_gnutls_parent_class)->complete_handshake (tls, handshake_succeeded, negotiated_protocol, error);
 
   /* It may have changed during the handshake, but we have to wait until here
    * because we can't emit notifies on the handshake thread.
    */
   if (gnutls->accepted_cas_changed)
     g_object_notify (G_OBJECT (gnutls), "accepted-cas");
+
+  if (handshake_succeeded)
+    {
+      /* If we're not using TLS 1.3, store the session ticket here. We
+       * don't normally perform session resumption in TLS 1.2, but we still
+       * support it if the application calls copy_session_state() (which
+       * doesn't exist for DTLS, so do this for TLS only).
+       *
+       * Note to distant future: remove this when dropping TLS 1.2 support.
+       */
+      session = g_tls_connection_gnutls_get_session (G_TLS_CONNECTION_GNUTLS (tls));
+      version = gnutls_protocol_get_version (session);
+      if (version <= GNUTLS_TLS1_2 && !g_tls_connection_base_is_dtls (tls))
+        {
+          gnutls_datum_t session_datum;
+
+          if (gnutls_session_get_data2 (g_tls_connection_gnutls_get_session (G_TLS_CONNECTION_GNUTLS (tls)),
+                                        &session_datum) == 0)
+            {
+              g_clear_pointer (&gnutls->session_data, g_bytes_unref);
+              gnutls->session_data = g_bytes_new_with_free_func (session_datum.data,
+                                                                 session_datum.size,
+                                                                 (GDestroyNotify)gnutls_free,
+                                                                 session_datum.data);
+            }
+        }
+  }
 }
 
 static void
@@ -515,6 +545,7 @@ g_tls_client_connection_gnutls_copy_session_state (GTlsClientConnection *conn,
   /* Precondition: source has handshaked, conn has not. */
   g_return_if_fail (!gnutls->session_id);
   g_return_if_fail (gnutls_source->session_id);
+  g_return_if_fail (!gnutls->session_data);
 
   /* Prefer to use a new session ticket, if possible. */
   gnutls->session_data = g_tls_backend_gnutls_lookup_session_data (gnutls_source->session_id);
@@ -522,9 +553,9 @@ g_tls_client_connection_gnutls_copy_session_state (GTlsClientConnection *conn,
   if (!gnutls->session_data && gnutls_source->session_data)
     {
       /* If it's not possible, we'll try to reuse the old ticket, even though
-       * this is a privacy risk since TLS 1.3. Applications should not use this
-       * function unless they need us to try as hard as possible to resume a
-       * session, even at the cost of privacy.
+       * this is a privacy risk. Applications should not use this function
+       * unless they need us to try as hard as possible to resume a session,
+       * even at the cost of privacy.
        */
       gnutls->session_data = g_bytes_ref (gnutls_source->session_data);
     }
